'' =================================================================================================
''
''   File....... jm_usd_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con

  RX1    = 31                                                   ' programming / terminal
  TX1    = 30
  
  SDA    = 29                                                   ' eeprom / i2c
  SCL    = 28

  SD_CS  = 25                                                   ' sd card connections
  SD_DI  = 24
  SD_CLK = 23
  SD_DO  = 22


con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial"
  sd   : "fsrw"
 

var

  byte  buffer[81]


dat

  TestFile              byte    "jonnymac.txt", 0
  
  TestStr               byte    "JonnyMac rocks the Propeller!", 0


pub main 

  term.start(RX1, TX1, %0000, 115_200)
  pause(10)

  term.tx(CLS)

  if (mount_sd)
    usd_demo
  else
    term.str(string("Could not mount SD card."))

  unmount_sd
  
  repeat
    waitcnt(0)


pub usd_demo | check

   if (open_file(@TestFile, "r"))
     term.str(string("Opening: "))
     term.str(@TestFile)
     term.tx(CR)
     term.tx(CR)
     repeat                                                     ' display contents
       check := readln(@buffer, 80)
       if (check < 0)
         quit
       else
         term.str(@buffer)

     open_file(@TestFile, "d")                                  ' delete for next run

   else
     term.str(string("File not found -- creating."))
     open_file(@TestFile, "w")
     repeat 3
       writeln(@TestStr, true, true)  
     close_file
     
  
con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con

  { ----------------- }
  {                   }
  {  F I L E   I / 0  } 
  {                   }
  { ----------------- }


var

  long  mounted
  

pub mount_sd | check

'' Attempt to mount SD card
'' -- returns true if SD card successfully mounted

  'check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)

  check := \sd.mount(SD_DO)

  if (check => 0)
    mounted := true
  else
    mounted := false

  return mounted


pub unmount_sd

'' Unmounts SD card

  \sd.unmount
  mounted := false


pub open_file(spntr, mode) | check

'' Attempt to open file
'' -- spntr is pointer to filename (z-string)
'' -- mode is single character, (e.g., "r" for read)
'' -- sd card must be mounted before open

  if (mounted)
    check := \sd.popen(spntr, mode)                             ' attempt to open
    if (check => 0)
      if ((mode == "r") or (mode == "R"))
        \sd.seek(0)                                             ' force sector read 
      return true
    else
      return false
  else
    return false


pub close_file

'' Closes open file

  \sd.pclose


pub readln(p_buf, n) | c, len

'' Reads line of text from open text file
'' -- terminated by LF or EOF
'' -- spntr is address of string buffer
'' -- n is maximum number of characters to read from line

  len := 0                                                     ' index into string

  repeat
    c := \sd.pgetc                                              ' get a character
    if (c < 0)                                                  ' end of file
      if (len == 0)                                             ' if nothing
        return -1                                               ' return empty
      else
        quit

    if (c <> 10)                                                ' if not EOL char
      byte[p_buf++] := c                                        '  add to buffer
      len++
    else                                                        ' if LF
      quit                                                      '  we're done

  byte[p_buf] := 0                                              ' terminate end of line

  return len
  

pub writeln(p_str, usecr, uself)

'' Writes z-string to open text file
'' -- set usecr to true to add CR
'' -- set uself to true to add LF

  \sd.pputs(p_str)
  if (usecr)
    \sd.pputc(13)
  if (uself)
    \sd.pputc(10)


pub readbuf(p_buf, n) | check

'' Reads n bytes from file to buffer at p_buf
'' -- returns true if good read

  check := \sd.pread(p_buf, n)
  if (check == n)
    return true
  else
    return false 


pub writebuf(p_buf, n) | check   

'' Writes n bytes to file from buffer at bpntr
'' -- returns true if good read

  check := \sd.pwrite(p_buf, n)
  if (check == n)
    return true
  else
    return false

      
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}